@@ -3,11 +3,10 @@ A low-level PDF generator.
 
 """
 
-import sys
 import zlib
 from codecs import BOM_UTF16_BE
 
-VERSION = __version__ = '0.0.2'
+VERSION = __version__ = '0.1.0'
 
 
 def _to_bytes(item):
@@ -17,13 +16,13 @@ def _to_bytes(item):
     elif isinstance(item, Object):
         return item.data
     elif isinstance(item, float):
-        if item.as_integer_ratio()[1] == 1:
-            return f'{int(item):d}'.encode('ascii')
+        if item.is_integer():
+            return f'{int(item):d}'.encode()
         else:
-            return f'{item:f}'.encode('ascii')
+            return f'{item:f}'.encode()
     elif isinstance(item, int):
-        return f'{item:d}'.encode('ascii')
-    return str(item).encode('ascii')
+        return f'{item:d}'.encode()
+    return str(item).encode()
 
 
 class Object:
@@ -43,8 +42,8 @@ class Object:
     def indirect(self):
         """Indirect representation of an object."""
         return b'\n'.join((
-            str(self.number).encode('ascii') + b' ' +
-            str(self.generation).encode('ascii') + b' obj',
+            str(self.number).encode() + b' ' +
+            str(self.generation).encode() + b' obj',
             self.data,
             b'endobj',
         ))
@@ -53,8 +52,8 @@ class Object:
     def reference(self):
         """Object identifier."""
         return (
-            str(self.number).encode('ascii') + b' ' +
-            str(self.generation).encode('ascii') + b' R')
+            str(self.number).encode() + b' ' +
+            str(self.generation).encode() + b' R')
 
     @property
     def data(self):
@@ -267,6 +266,10 @@ class Stream(Object):
         self.stream.append(
             b'/' + _to_bytes(font) + b' ' + _to_bytes(size) + b' Tf')
 
+    def set_text_rendering(self, mode):
+        """Set text rendering mode."""
+        self.stream.append(_to_bytes(mode) + b' Tr')
+
     def set_line_cap(self, line_cap):
         """Set line cap style."""
         self.stream.append(_to_bytes(line_cap) + b' J')
@@ -375,7 +378,7 @@ class String(Object):
             return b'(' + _to_bytes(self.string) + b')'
         except UnicodeEncodeError:
             encoded = BOM_UTF16_BE + str(self.string).encode('utf-16-be')
-            return b'<' + encoded.hex().encode('ascii') + b'>'
+            return b'<' + encoded.hex().encode() + b'>'
 
 
 class Array(Object, list):
@@ -454,61 +457,46 @@ class PDF:
         :param content: Content to write.
         :type content: :obj:`bytes`
         :param output: Output stream.
-        :type output: :term:`file object`
+        :type output: binary :term:`file object`
 
         """
         self.current_position += len(content) + 1
         output.write(content + b'\n')
 
-    def write_object(self, object_, output):
-        """Write object to output."""
-        for line in object_.data.split(b'\n'):
-            self.write_line(line, output)
+    def write(self, output):
+        """Write PDF to output.
 
-    def write_header(self, output):
-        """Write PDF header to output."""
+        :param output: Output stream.
+        :type output: binary :term:`file object`
+
+        """
+        # Write header
         self.write_line(b'%PDF-1.7', output)
         self.write_line(b'%\xf0\x9f\x96\xa4', output)
 
-    def write_body(self, output):
-        """Write all non-free PDF objects to output."""
+        # Write all non-free PDF objects
         for object_ in self.objects:
             if object_.free == 'f':
                 continue
             object_.offset = self.current_position
             self.write_line(object_.indirect, output)
 
-    def write_cross_reference_table(self, output):
-        """Write cross reference table to output."""
+        # Write cross reference table
         self.xref_position = self.current_position
         self.write_line(b'xref', output)
-        self.write_line(f'0 {len(self.objects)}'.encode('ascii'), output)
+        self.write_line(f'0 {len(self.objects)}'.encode(), output)
         for object_ in self.objects:
             self.write_line(
                 (f'{object_.offset:010} {object_.generation:05} '
-                 f'{object_.free} ').encode('ascii'), output
-            )
+                 f'{object_.free} ').encode(), output)
 
-    def write_trailer(self, output):
-        """Write trailer to output."""
+        # Write trailer
         self.write_line(b'trailer', output)
-        self.write_object(Dictionary({
-            'Size': len(self.objects),
-            'Root': self.catalog.reference,
-            'Info': self.info.reference,
-        }), output)
+        self.write_line(b'<<', output)
+        self.write_line(f'/Size {len(self.objects)}'.encode(), output)
+        self.write_line(b'/Root ' + self.catalog.reference, output)
+        self.write_line(b'/Info ' + self.info.reference, output)
+        self.write_line(b'>>', output)
         self.write_line(b'startxref', output)
-        self.write_line(str(self.xref_position).encode('ascii'), output)
+        self.write_line(f'{self.xref_position}'.encode(), output)
         self.write_line(b'%%EOF', output)
-
-    def write(self, output=sys.stdout.buffer):
-        """Write PDF to output.
-
-        :param output: Output stream, :obj:`sys.stdout` by default.
-        :type output: :term:`file object`
-
-        """
-        self.write_header(output)
-        self.write_body(output)
-        self.write_cross_reference_table(output)
-        self.write_trailer(output)
